@@ -18,10 +18,9 @@ import time
 import cgi
 import os
 import shutil
-import webbrowser
 import smtplib
 import shlex
-from Cookie import SimpleCookie
+from Cookie import BaseCookie
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -208,6 +207,41 @@ class TestApp(object):
         req = TestRequest(url, environ, expect_errors)
         return self.do_request(req, status=status)
 
+    def _gen_request(self, method, url, params='', headers=None, extra_environ=None,
+             status=None, upload_files=None, expect_errors=False):
+        """
+        Do a generic request.  
+        """
+        if headers is None:
+            headers = {}
+        if extra_environ is None:
+            extra_environ = {}
+        environ = self._make_environ()
+        # @@: Should this be all non-strings?
+        if isinstance(params, (list, tuple, dict)):
+            params = urllib.urlencode(params)
+        if hasattr(params, 'items'):
+            # Some other multi-dict like format
+            params = urllib.urlencode(params.items())
+        if upload_files:
+            params = cgi.parse_qsl(params, keep_blank_values=True)
+            content_type, params = self.encode_multipart(
+                params, upload_files)
+            environ['CONTENT_TYPE'] = content_type
+        elif params:
+            environ.setdefault('CONTENT_TYPE', 'application/x-www-form-urlencoded')
+        if '?' in url:
+            url, environ['QUERY_STRING'] = url.split('?', 1)
+        else:
+            environ['QUERY_STRING'] = ''
+        environ['CONTENT_LENGTH'] = str(len(params))
+        environ['REQUEST_METHOD'] = method
+        environ['wsgi.input'] = StringIO(params)
+        self._set_headers(headers, environ)
+        environ.update(extra_environ)
+        req = TestRequest(url, environ, expect_errors)
+        return self.do_request(req, status=status)
+
     def post(self, url, params='', headers=None, extra_environ=None,
              status=None, upload_files=None, expect_errors=False):
         """
@@ -222,30 +256,45 @@ class TestApp(object):
         Returns a `response object
         <class-paste.fixture.TestResponse.html>`_
         """
-        if headers is None:
-            headers = {}
-        if extra_environ is None:
-            extra_environ = {}
-        environ = self._make_environ()
-        # @@: Should this be all non-strings?
-        if isinstance(params, (list, tuple, dict)):
-            params = urllib.urlencode(params)
-        if upload_files:
-            params = cgi.parse_qsl(params, keep_blank_values=True)
-            content_type, params = self.encode_multipart(
-                params, upload_files)
-            environ['CONTENT_TYPE'] = content_type
-        if '?' in url:
-            url, environ['QUERY_STRING'] = url.split('?', 1)
-        else:
-            environ['QUERY_STRING'] = ''
-        environ['CONTENT_LENGTH'] = str(len(params))
-        environ['REQUEST_METHOD'] = 'POST'
-        environ['wsgi.input'] = StringIO(params)
-        self._set_headers(headers, environ)
-        environ.update(extra_environ)
-        req = TestRequest(url, environ, expect_errors)
-        return self.do_request(req, status=status)
+        return self._gen_request('POST', url, params=params, headers=headers,
+                                 extra_environ=extra_environ,status=status,
+                                 upload_files=upload_files,
+                                 expect_errors=expect_errors)
+
+    def put(self, url, params='', headers=None, extra_environ=None,
+             status=None, upload_files=None, expect_errors=False):
+        """
+        Do a PUT request.  Very like the ``.get()`` method.
+        ``params`` are put in the body of the request.
+
+        ``upload_files`` is for file uploads.  It should be a list of
+        ``[(fieldname, filename, file_content)]``.  You can also use
+        just ``[(fieldname, filename)]`` and the file content will be
+        read from disk.
+
+        Returns a `response object
+        <class-paste.fixture.TestResponse.html>`_
+        """
+        return self._gen_request('PUT', url, params=params, headers=headers,
+                                 extra_environ=extra_environ,status=status,
+                                 upload_files=upload_files,
+                                 expect_errors=expect_errors)
+
+    def delete(self, url, params='', headers=None, extra_environ=None,
+               status=None, expect_errors=False):
+        """
+        Do a DELETE request.  Very like the ``.get()`` method.
+        ``params`` are put in the body of the request.
+
+        Returns a `response object
+        <class-paste.fixture.TestResponse.html>`_
+        """
+        return self._gen_request('DELETE', url, params=params, headers=headers,
+                                 extra_environ=extra_environ,status=status,
+                                 upload_files=None, expect_errors=expect_errors)
+
+    
+
 
     def _set_headers(self, headers, environ):
         """
@@ -320,10 +369,11 @@ class TestApp(object):
             self.pre_request_hook(self)
         __tracebackhide__ = True
         if self.cookies:
-            c = SimpleCookie()
+            c = BaseCookie()
             for name, value in self.cookies.items():
                 c[name] = value
-            req.environ['HTTP_COOKIE'] = str(c).split(': ', 1)[1]
+            hc = '; '.join(['='.join([m.key, m.value]) for m in c.values()])
+            req.environ['HTTP_COOKIE'] = hc
         req.environ['paste.testing'] = True
         req.environ['paste.testing_variables'] = {}
         app = lint.middleware(self.app)
@@ -357,7 +407,7 @@ class TestApp(object):
             self._check_errors(res)
         res.cookies_set = {}
         for header in res.all_headers('set-cookie'):
-            c = SimpleCookie(header)
+            c = BaseCookie(header)
             for key, morsel in c.items():
                 self.cookies[key] = morsel.value
                 res.cookies_set[key] = morsel.value
@@ -381,8 +431,7 @@ class TestApp(object):
                        res.request.url, res.body))
             return
         if status is None:
-            if res.status == 200 or (
-                res.status >= 300 and res.status < 400):
+            if res.status >= 200 and res.status < 400:
                 return
             raise AppError(
                 "Bad response: %s (not 200 OK or 3xx redirect for %s)\n%s"
@@ -461,10 +510,13 @@ class TestResponse(object):
 
     def form__get(self):
         forms = self.forms
-        assert len(forms) > 0, (
-            "You used response.form, but no forms exist")
-        assert len(forms) == 1, (
-            "You used response.form, but more than one form exists")
+        if not forms:
+            raise TypeError(
+                "You used response.form, but no forms exist")
+        if 1 in forms:
+            # There is more than one form
+            raise TypeError(
+                "You used response.form, but more than one form exists")
         return forms[0]
 
     form = property(form__get,
@@ -475,7 +527,7 @@ class TestResponse(object):
                     page.
                     """)
 
-    _tag_re = re.compile(r'<(/?)([a-z0-9_\-]*)(.*?)>', re.S|re.I)
+    _tag_re = re.compile(r'<(/?)([:a-z0-9_\-]*)(.*?)>', re.S|re.I)
 
     def _parse_forms(self):
         forms = self._forms_indexed = {}
@@ -740,6 +792,9 @@ class TestResponse(object):
         """
         if not isinstance(s, (str, unicode)):
             s = str(s)
+        if isinstance(s, unicode):
+            ## FIXME: we don't know that this response uses utf8:
+            s = s.encode('utf8')
         return (self.body.find(s) != -1
                 or self.normal_body.find(s) != -1)
 
@@ -791,6 +846,7 @@ class TestResponse(object):
         Show this response in a browser window (for debugging purposes,
         when it's hard to read the HTML).
         """
+        import webbrowser
         fn = tempnam_no_warning(None, 'paste-fixture') + '.html'
         f = open(fn, 'wb')
         f.write(self.body)
@@ -863,7 +919,7 @@ class Form(object):
     # @@: This really should be using Mechanize/ClientForm or
     # something...
 
-    _tag_re = re.compile(r'<(/?)([a-z0-9_\-]*)([^>]*?)>', re.I)
+    _tag_re = re.compile(r'<(/?)([:a-z0-9_\-]*)([^>]*?)>', re.I)
 
     def __init__(self, response, text):
         self.response = response
@@ -1046,6 +1102,8 @@ class Form(object):
             field = self.get(name, index=index)
             submit.append((field.name, field.value_if_submitted()))
         for name, fields in self.fields.items():
+            if name is None:
+                continue
             for field in fields:
                 value = field.value
                 if value is None:
@@ -1172,8 +1230,7 @@ class Checkbox(Field):
     def value__get(self):
         if self.checked:
             if self._value is None:
-                # @@: 'on'?
-                return 'checked'
+                return 'on'
             else:
                 return self._value
         else:
@@ -1187,7 +1244,12 @@ class Text(Field):
     """
     Field representing ``<input type="text">``
     """
-
+    def __init__(self, form, tag, name, pos,
+                 value='', id=None, **attrs):
+        #text fields default to empty string        
+        Field.__init__(self, form, tag, name, pos,
+                       value=value, id=id, **attrs)
+                        
 Field.classes['text'] = Text
 
 class Textarea(Text):
@@ -1223,6 +1285,8 @@ Field.classes['submit'] = Submit
 
 Field.classes['button'] = Submit
 
+Field.classes['image'] = Submit
+
 ############################################################
 ## Command-line testing
 ############################################################
@@ -1462,7 +1526,7 @@ class ProcResult(object):
 
     def assert_no_error(self):
         __tracebackhide__ = True
-        assert self.returncode is 0, (
+        assert self.returncode == 0, (
             "Script returned code: %s" % self.returncode)
 
     def assert_no_stderr(self):
@@ -1633,8 +1697,13 @@ def setup_module(module=None):
 
     Use like::
 
-        from paste.tests.fixture import setup_module
+        from paste.fixture import setup_module
     """
+    # Deprecated June 2008
+    import warnings
+    warnings.warn(
+        'setup_module is deprecated',
+        DeprecationWarning, 2)
     if module is None:
         # The module we were called from must be the module...
         module = sys._getframe().f_back.f_globals['__name__']
