@@ -913,7 +913,7 @@ class _Range(_MultiValueHeader):
         ranges = []
         last_end   = -1
         try:
-            (units, range) = value.split("=")
+            (units, range) = value.split("=", 1)
             units = units.strip().lower()
             for item in range.split(","):
                 (begin, end) = item.split("-")
@@ -941,7 +941,7 @@ class _AcceptLanguage(_MultiValueHeader):
     """
     Accept-Language, RFC 2616 section 14.4
     """
-    languageRegEx = re.compile(r"^[a-z]{2}(-[a-z]{2})?$", re.I)
+
     def parse(self, *args, **kwargs):
         """
         Return a list of language tags sorted by their "q" values.  For example,
@@ -951,15 +951,16 @@ class _AcceptLanguage(_MultiValueHeader):
         header = self.__call__(*args, **kwargs)
         if header is None:
             return []
-        langs = header.split(",")
+        langs = [v for v in header.split(",") if v]
         qs = []
         for lang in langs:
             pieces = lang.split(";")
             lang, params = pieces[0].strip().lower(), pieces[1:]
-            if not self.languageRegEx.match(lang):
-                continue
             q = 1
             for param in params:
+                if '=' not in param:
+                    # Malformed request; probably a bot, we'll ignore
+                    continue
                 lvalue, rvalue = param.split("=")
                 lvalue = lvalue.strip().lower()
                 rvalue = rvalue.strip()
@@ -985,7 +986,7 @@ class _ContentRange(_SingleValueHeader):
     Content-Range, RFC 2616 section 14.6
     """
     def compose(self, first_byte=None, last_byte=None, total_length=None):
-        retval = "%d-%d/%d" % (first_byte, last_byte, total_length)
+        retval = "bytes %d-%d/%d" % (first_byte, last_byte, total_length)
         assert last_byte == -1 or first_byte <= last_byte
         assert last_byte  < total_length
         return (retval,)
